;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
; Project name : Tutorial
; Author : Fabrice SINCERE
; Date : 02/06
; Microcontroller : Microchip PIC 16F84A
; Language : Assembler
; IDE : Microchip MPLAB
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;xxxxxxxxxxx
;  Processor
;xxxxxxxxxxx

	List p=16F84A
	#include <p16F84A.inc>

;xxxxxxxxxxxxxxxxxxxxxx
; Configuration bits
;xxxxxxxxxxxxxxxxxxxxxx

	__config _CP_OFF & _WDT_OFF & _PWRTE_ON & _RC_OSC

	;code protect OFF
	;watchdog timer OFF
	;power up timer ON
	;oscillator RC

;xxxxxx
; Macro
;xxxxxx

bank1	macro	; select bank 1
	bsf STATUS,RP0
	endm

bank0	macro	; select bank 0
	bcf STATUS,RP0
	endm

;xxxxxxxxxxxxxxxxxxxxxxxxxxx
; EEPROM initialization
;xxxxxxxxxxxxxxxxxxxxxxxxxxx

	org 0x2100	; EEPROM absolute adress
	DE 0x50	; adress 0x00
	DE 0x69	; adress 0x01
	DE 0x63	; adress 0x02
	DE 0x69	; adress 0x03
	DE 0x6E	; adress 0x04
	DE 0x69	; adress 0x05
	DE 0x74	; adress 0x06
	DE 0x00	; adress 0x07
	DE 0x00	; adress 0x08
	DE 0x00	; adress 0x09
	DE 0x00	; adress 0x0A
	DE 0x00	; adress 0x0B
	DE 0x00	; adress 0x0C
	DE 0x00	; adress 0x0D
	DE 0x00	; adress 0x0E
	DE 0x00	; adress 0x0F
	DE 0x00	; adress 0x10
	DE 0x00	; adress 0x11
	DE 0x00	; adress 0x12
	DE 0x00	; adress 0x13
	DE 0x00	; adress 0x14
	DE 0x00	; adress 0x15
	DE 0x00	; adress 0x16
	DE 0x00	; adress 0x17
	DE 0x00	; adress 0x18
	DE 0x00	; adress 0x19
	DE 0x00	; adress 0x1A
	DE 0x00	; adress 0x1B
	DE 0x00	; adress 0x1C
	DE 0x00	; adress 0x1D
	DE 0x00	; adress 0x1E
	DE 0x00	; adress 0x1F
	DE 0x00	; adress 0x20
	DE 0x00	; adress 0x21
	DE 0x00	; adress 0x22
	DE 0x00	; adress 0x23
	DE 0x00	; adress 0x24
	DE 0x00	; adress 0x25
	DE 0x00	; adress 0x26
	DE 0x00	; adress 0x27
	DE 0x00	; adress 0x28
	DE 0x00	; adress 0x29
	DE 0x00	; adress 0x2A
	DE 0x00	; adress 0x2B
	DE 0x00	; adress 0x2C
	DE 0x00	; adress 0x2D
	DE 0x00	; adress 0x2E
	DE 0x00	; adress 0x2F
	DE 0x00	; adress 0x30
	DE 0x00	; adress 0x31
	DE 0x00	; adress 0x32
	DE 0x00	; adress 0x33
	DE 0x00	; adress 0x34
	DE 0x00	; adress 0x35
	DE 0x00	; adress 0x36
	DE 0x00	; adress 0x37
	DE 0x00	; adress 0x38
	DE 0x00	; adress 0x39
	DE 0x00	; adress 0x3A
	DE 0x00	; adress 0x3B
	DE 0x00	; adress 0x3C
	DE 0x00	; adress 0x3D
	DE 0x00	; adress 0x3E
	DE 0xAA	; adress 0x3F
	
;xxxxxxxxxxxxxxxxxxxxxxxxxxx
; General purpose registers
;xxxxxxxxxxxxxxxxxxxxxxxxxxx

	CBLOCK 0x0C

	W_TEMP : 1 	; adress 0x0C
	STATUS_TEMP : 1 	; adress 0x0D
	count_1 : 1 	; adress 0x0E
	count_2 : 1 	; adress 0x0F
	count_3 : 1 	; adress 0x10
	index_1 : 1 	; adress 0x11
	index_2 : 1 	; adress 0x12
	index_3 : 1 	; adress 0x13
	
	ENDC

;xxxxxxxxxxxxxxxxxxxx
; Reset vector
;xxxxxxxxxxxxxxxxxxxx

	org 0x0000
	goto initialization

;xxxxxxxxxxxxxxxxxxxxxxx
; Interrupt routine
;xxxxxxxxxxxxxxxxxxxxxxx

	org 0x0004	; interrupt vector
	movwf W_TEMP	; stores W and STATUS
	swapf STATUS,W
	movwf STATUS_TEMP

	; Interrupt routine

	swapf STATUS_TEMP,W	; restores STATUS and W
	movwf STATUS
	swapf W_TEMP,f
	swapf W_TEMP,W
	retfie	; return from interrupt

;xxxxxxxxxxxxxxx
; Initialization
;xxxxxxxxxxxxxxx

initialization

	bank0
	clrf PORTA
	clrf PORTB

	bank1

	movlw B'00010100'
	movwf OPTION_REG
	; bit 7 (/RBPU) = 0 : PORTB pull-ups are enabled
	; bit 6 (INTEDG) = 0 : Interrupt on falling edge of RB0/INT pin
	; bit 5 (T0CS) = 0 : Internal instruction cycle clock
	; bit 4 (T0SE) = 1 : option not used (1 by default)

	; bit 3 (PSA) = 0
	; bit 2 (PS2) = 1
	; bit 1 (PS1) = 0
	; bit 0 (PS0) = 0
	; Timer0 Rate = 1:32
	; Watchdog Rate = 1:1

	movlw B'00000111'
	movwf TRISA
	; port A bit 7 : unimplemented
	; port A bit 6 : unimplemented
	; port A bit 5 : unimplemented
	; port A bit 4 (RA4/T0CKI) = 0 : configured as an output
	; port A bit 3 (RA3) = 0 : configured as an output
	; port A bit 2 (RA2) = 1 : configured as an input
	; port A bit 1 (RA1) = 1 : not connected (by default, pin is configured as an input)
	; port A bit 0 (RA0) = 1 : configured as an input
	
	movlw B'00111001'
	movwf TRISB
	; port B bit 7 (RB7) = 0 : configured as an output
	; port B bit 6 (RB6) = 0 : configured as an output
	; port B bit 5 (RB5) = 1 : configured as an input
	; port B bit 4 (RB4) = 1 : configured as an input
	; port B bit 3 (RB3) = 1 : not connected (by default, pin is configured as an input)
	; port B bit 2 (RB2) = 0 : configured as an output
	; port B bit 1 (RB1) = 0 : configured as an output
	; port B bit 0 (RB0/INT) = 1 : configured as an input

	bank0

	movlw B'00001000'
	movwf PORTA
	; Output : RA4/T0CKI = 0
	; Output : RA3 = 1
	
	movlw B'10000100'
	movwf PORTB
	; Output : RB7 = 1
	; Output : RB6 = 0
	; Output : RB2 = 1
	; Output : RB1 = 0
	
	clrf count_1
	movlw 0x0E
	movwf count_3
	movlw 0xF3
	movwf index_1
	movlw 0xF4
	movwf index_2
	clrf index_3
	
	clrf TMR0

	movlw B'10110000'
	movwf INTCON
	; bit 7 (GIE) = 1 : Enables all unmasked interrupts
	; bit 6 (EEIE) = 0 : Disables the EE Write Complete interrupt
	; bit 5 (T0IE) = 1 : Enables the TMR0 interrupt
	; bit 4 (INTE) = 1 : Enables the RB0/INT external interrupt
	; bit 3 (RBIE) = 0 : Disables the RB port change interrupt
	; bit 2 (T0IF) = 0 : clear TMR0 Overflow Interrupt Flag bit
	; bit 1 (INTF) = 0 : clear RB0/INT External Interrupt Flag bit
	; bit 0 (RBIF) = 0 : clear RB Port Change Interrupt Flag bit

	goto main_program

;xxxxxxxxxxxxxxxxxxxx
; Main Program
;xxxxxxxxxxxxxxxxxxxx

main_program

	; Main Program

	goto main_program

	END